import java.awt.*;
import java.awt.event.*;

/**
*	This class provides an interface for entering data via a textfield.
*	The variable's value is automatically set to the Double.parseDouble value
*	of the inputField when focus is lost.
*	If the inputField is unable to be parsed as a double, the value is set
*	0.0 when focus is lost.
*
*
*	@author Rob Falck - Case Western Reserve University
**/

public class Variable implements FocusListener {

	/** The numerical value of this variable **/
	public double value;

	/** The input interface for this variable **/
	public TextField inputField;

	/** A text label for this variable **/
	Label label = new Label(null,Label.CENTER);

	/** Maximum value (this.value <= maximum value) **/
	public double maxValue= Double.POSITIVE_INFINITY;

	/** Minimum value (this.value >= minimum value) **/
	public double minValue=Double.NEGATIVE_INFINITY;


	/** Create a new variable with the specified value **/
	public Variable(double value) {
		inputField = new TextField(6);
		inputField.addFocusListener(this);
		this.setValue(value);
	}

	/** Create a new variable with the specified description **/
	public Variable(String description) {
		inputField = new TextField(6);
		inputField.addFocusListener(this);
		this.label.setText(description);
	}

	/** Create a new variable with the specified description and value **/
	public Variable(String description, double value) {
		inputField = new TextField(6);
		inputField.addFocusListener(this);
		this.label.setText(description);
		this.setValue(value);
	}

	/** Create a new variable with an initial value of 0.0 **/
	public Variable() {
		inputField = new TextField(6);
		inputField.addFocusListener(this);
	}

	/**
	*	Set the value of this variable to the parsed double value of the given string.
	*	Value is set to 0.0 if given string cannot be parsed as a double.
	**/
	public void setValue(String inputValue) {
		inputField.setText(inputValue);
		try {
			this.value = Double.valueOf(inputValue).doubleValue();
		} catch (NumberFormatException exc) {
			this.setValue(0);
		}
	}

	/** Set the value of this variable to the given double value **/
	public void setValue(double value) {
		inputField.setText(Double.toString(value));
		this.value = value;
	}

	/** Set the value of this variable to the value in its textfield.
	*	Defaults to 0.0 if the value is not a valid double.
	**/
	public void setValue() {
		if(this.value < this.minValue) {
			this.setValue(this.minValue);
			return;
		} else if(this.value > this.maxValue) {
			this.setValue(this.maxValue);
			return;
		}
		try {
			this.value = Double.valueOf(this.inputField.getText()).doubleValue();
		} catch (NumberFormatException exc) {
			this.setValue(0);
		}
	}



	/** Returns the numeric value of this variable as a double **/
	public double getValue() {
		return(value);
	}

	/** Set the maximum possible value for this variable **/
	public void setMaxValue(double max) {
		this.maxValue = max;
	}

	/** Set the minimum possible value for this variable **/
	public void setMinValue(double min) {
		this.minValue = min;
	}


	/** Get the maximum possible value for this variable **/
	public double getMaxValue() {
		return(this.maxValue);
	}

	/** Get the minimum possible value for this variable **/
	public double getMinValue() {
		return(this.minValue);
	}


	/** FocusListenter method to alert this object that input focus has been acquired **/
	public void focusGained(FocusEvent e) {
	}

	/** FocusListenter method to alert this object that input focus has been lost
	*	Note that this method makes a call specific to the Cosmo applet and will
	*	therefore need to be altered before it can be used in other applets.
	**/
    public void focusLost(FocusEvent e) {
		this.setValue();
		Cosmo.dataIsNew = true;
	}




}
